@@ -219,10 +219,6 @@ class BaseWritter(Writer):
         cflags = opts.cflags.copy()
         post_cflags = opts.post_cflags
         if pch:
-            # remove_flags = ["-fPIC", "-O3", "-std=c++14"]
-            # for f in remove_flags:
-            #     if f in cflags:
-            #         cflags.remove(f)
             cflags.append("-x")
             cflags.append("c++-header")
 
@@ -239,19 +235,14 @@ class BaseWritter(Writer):
         compile_stmt = "${} -MMD -MT $out -MF $out.d {} {} -c $in -o $out {}"
         if use_pch:
             compile_stmt = "${} -MMD -MT $out -MF $out.d {} {} -include $pch -c $in -o $out {}"
-        if pch:
-            compile_stmt = "${} {} {} -c $in -o $out {}"
-            self.rule(rule_name,
-                      compile_stmt.format(compiler_var, includes, cflags,
-                                          post_cflags),
-                      description=desc)
-        else:
-            self.rule(rule_name,
-                      compile_stmt.format(compiler_var, includes, cflags,
-                                          post_cflags),
-                      description=desc,
-                      depfile="$out.d",
-                      deps="gcc")
+        # if pch:
+        #     compile_stmt = "${} {} {} -c $in -o $out {}"
+        self.rule(rule_name,
+                  compile_stmt.format(compiler_var, includes, cflags,
+                                      post_cflags),
+                  description=desc,
+                  depfile="$out.d",
+                  deps="gcc")
 
         self.newline()
         return rule_name
@@ -318,17 +309,11 @@ class BaseWritter(Writer):
             desc = "[MSVC][c++/pch]$pchobj|$out"
         if use_pch:
             compile_stmt = "${} {} {} /nologo /showIncludes -c /Yu$pch /Fp$pchobj $in /Fo$out {}"
-            self.rule(rule_name,
-                      compile_stmt.format(compiler_var, includes, cflags,
-                                          post_cflags),
-                      deps="msvc",
-                      description=desc)
-        else:
-            self.rule(rule_name,
-                      compile_stmt.format(compiler_var, includes, cflags,
-                                          post_cflags),
-                      deps="msvc",
-                      description=desc)
+        self.rule(rule_name,
+                  compile_stmt.format(compiler_var, includes, cflags,
+                                      post_cflags),
+                  deps="msvc",
+                  description=desc)
 
         self.newline()
         return rule_name
@@ -364,6 +349,8 @@ class BaseWritter(Writer):
         includes = " ".join(
             ["-I \"{}\"".format(str(i)) for i in opts.includes])
         cflags = opts.cflags
+        if "-keep" in cflags or "--keep" in cflags:
+            cflags.append("--keep-dir $builddir")
         post_cflags = opts.post_cflags
         cflags = " ".join(cflags)
         post_cflags = " ".join(post_cflags)
@@ -478,7 +465,10 @@ class BaseWritter(Writer):
                                                  str]] = None):
         source_paths = [_unify_path(p) for p in sources]
         if pch_to_include is not None:
-            pch_to_include = {_unify_path(p): v for p, v in pch_to_include.items()}
+            pch_to_include = {
+                _unify_path(p): v
+                for p, v in pch_to_include.items()
+            }
         if pch_to_sources is None:
             pch_to_sources = {}
         unified_pch_to_sources = {}  # type: Dict[Path, List[Path]]
@@ -544,7 +534,8 @@ class BaseWritter(Writer):
                         stub_file = self._create_msvc_stub_path(
                             pch_path, name_pool)
                         write_stub = True
-                        stub_content = "#include <{}>".format(pch_to_include[pch_path])
+                        stub_content = "#include <{}>".format(
+                            pch_to_include[pch_path])
                         if stub_file.exists():
                             with stub_file.open("r") as f:
                                 data = f.read().strip()
@@ -631,18 +622,27 @@ class BaseWritter(Writer):
                                str(p),
                                variables={
                                    "pch": pch_to_include[Path(pch_path)],
-                                   "pchobj": pch_obj
+                                   "pchobj": pch_obj,
+                                   "builddir": str(p.parent),
                                },
                                implicit=[stub])
                 else:
                     self.build(obj_path,
                                rule,
                                str(p),
-                               variables={"pch": pch_path},
+                               variables={
+                                   "pch": pch_path,
+                                   "builddir": str(p.parent),
+                               },
                                implicit=[pch_obj])
 
             else:
-                self.build(obj_path, rule, str(p))
+                self.build(obj_path,
+                           rule,
+                           str(p),
+                           variables={
+                               "builddir": str(p.parent),
+                           })
 
         self.newline()
         stub_obj_files_list = list(stub_obj_files)
@@ -947,9 +947,17 @@ def run_simple_ninja(target,
                      link_opts=None,
                      target_filename=None,
                      additional_cflags=None):
-    ninja_content, target_filename = create_simple_ninja(
-        target, build_dir, sources, includes, libs, libpaths, compile_opts,
-        link_opts, target_filename, additional_cflags, False)
+    ninja_content, target_filename = create_simple_ninja(target,
+                                                         build_dir,
+                                                         sources,
+                                                         includes,
+                                                         libs,
+                                                         libpaths,
+                                                         compile_opts,
+                                                         link_opts,
+                                                         target_filename,
+                                                         additional_cflags,
+                                                         shared=False)
     build_dir = Path(build_dir)
     with (build_dir / "build.ninja").open("w") as f:
         f.write(ninja_content)
